/* m_services.c - Because s_user.c was just crazy.
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: m_services.c,v 1.10 2002/07/11 20:42:36 dgl Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <sys/stat.h>
#include <utmp.h>
#include <fcntl.h>
#include "h.h"

/* Externally defined stuffs */
extern int user_modes[];

/* internally defined stuffs */




/*
 * the services aliases. *
 *
 * NICKSERV     - /nickserv * CHANSERV  - /chanserv * OPERSERV  -
 * /operserv * MEMOSERV         - /memoserv * SERVICES  - /services *
 * IDENTIFY     - /identify * taz's code -mjs
 */

/* m_ns */
int m_dogtag(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    aClient    *acptr;

    if (check_registered_user(sptr))
	return 0;
    if (parc < 2 || *parv[1] == '\0')
    {
        if(MyClient(sptr))
	  sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    if ((acptr = find_server(SERVICES_NAME, NULL)))
#ifdef SERVICESHUB
	sendto_one(acptr, ":%s NS :%s", parv[0], parv[1]);
#else
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", 
		parv[0], NICKSERV, SERVICES_NAME, parv[1]);
#endif
    else
	sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name,
		   parv[0], NICKSERV, NICKSERV);
    return 0;
}

/* m_cs */
int m_k9(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    aClient    *acptr;

    if (check_registered_user(sptr))
	return 0;
    if (parc < 2 || *parv[1] == '\0')
    {
        if(MyClient(sptr))
	  sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    if ((acptr = find_server(SERVICES_NAME, NULL)))
#ifdef SERVICESHUB
	sendto_one(acptr, ":%s CS :%s", parv[0], parv[1]);
#else
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", 
		parv[0], CHANSERV, SERVICES_NAME, parv[1]);
#endif
    else
	sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name,
		   parv[0], CHANSERV, CHANSERV);
    return 0;
}

int m_cw(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    aClient    *acptr;

    if (check_registered_user(sptr))
	return 0;
    if (parc < 2 || *parv[1] == '\0')
    {
        if(MyClient(sptr))
	  sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    if ((acptr = find_server(CHATWORLD_NAME, NULL)))
#ifdef SERVICESHUB
	sendto_one(acptr, ":%s CW :%s", parv[0], parv[1]);
#else
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", 
		parv[0], CHATWORLD, CHATWORLD_NAME, parv[1]);
#endif
    else
	sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name,
		   parv[0], CHATWORLD, CHATWORLD);
    return 0;
}

/* m_os */
int m_os(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    aClient    *acptr;

    if (check_registered_user(sptr))
	return 0;
    if (parc < 2 || *parv[1] == '\0')
    {
        if(MyClient(sptr))
	  sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    if ((acptr = find_server(OS_NAME, NULL)))
#ifdef SERVICESHUB
	sendto_one(acptr, ":%s OS :%s", parv[0], parv[1]);
#else
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", 
		parv[0], OPERSERV, OS_NAME, parv[1]);
#endif
    else
	sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name,
		   parv[0], OPERSERV, OPERSERV);
    return 0;
}

/* m_services -- see df465+taz */
int m_services(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char       *tmps;

    if (check_registered_user(sptr))
	return 0;

    if (parc < 2 || *parv[1] == '\0')
    {
	sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    if ((strlen(parv[1]) >= 4) && (!myncmp(parv[1], "help", 4)))
    {
	sendto_one(sptr, ":K9!K9@%s NOTICE %s :For K9 "
		   "help use: /K9 help", SERVICES_NAME,
		   sptr->name);
	sendto_one(sptr, ":DogTag!K9@%s NOTICE %s :For DogTag "
		   "help use: /DogTag help", SERVICES_NAME,
		   sptr->name);
	sendto_one(sptr, ":ChatWorld!ChatWorld@%s NOTICE %s :For ChatWorld "
		   "help use: /ChatWorld help", CHATWORLD_NAME,
		   sptr->name);
	return 0;
    }
    if ((tmps = (char *) strchr(parv[1], ' ')))
    {
	for(; *tmps == ' '; tmps++); /* er.. before this for loop, the next
				      * comparison would always compare '#' 
				      * with ' '.. oops. - lucas
				      */
	if (*tmps == '#')
	    return m_k9(cptr, sptr, parc, parv);
	else
	    return m_dogtag(cptr, sptr, parc, parv);
    }
    return m_dogtag(cptr, sptr, parc, parv);
}

/* m_identify  df465+taz */
int m_identify(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char       buf[BUFSIZE+1];
    char       *myparv[parc];

    if (check_registered_user(sptr))
	return 0;

    if (parc < 2 || *parv[1] == '\0')
    {
	sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
	return -1;
    }
    (void) ircsnprintf(buf, BUFSIZE, "IDENTIFY %s", parv[1]);

    myparv[0]=parv[0];
    myparv[1]=buf;

    if (*parv[1] == '#')
      return m_k9(cptr, sptr, parc, myparv);
    else
      return m_dogtag(cptr, sptr, parc, myparv);

    return 0;
}

/* s_svsnick - Pretty straight forward.  Mostly straight outta df
 *  - Raistlin
 * parv[0] = sender
 * parv[1] = old nickname
 * parv[2] = new nickname
 * parv[3] = timestamp
 */
int m_svsnick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;

    if (!IsULine(sptr)||parc < 4||(strlen(parv[2]) > NICKLEN)) return 0;
    /* if we can't SVSNICK them to something because the nick 
     * is in use, KILL them
     */
    acptr=find_person(parv[2], NULL);
    if(acptr!=NULL)
    {
	/* send a kill out for the person instead --wd */
	sendto_serv_butone(cptr, ":%s KILL %s :%s (SVSNICK Collide)",
			   sptr->name, parv[1], sptr->name);
	/* now send back a kill for the nick they were presumably changed to */
	sendto_one(cptr, ":%s KILL %s :%s (SVSNICK Collide)",
		   sptr->name, parv[2], sptr->name);
	return 0;
    }
    if (!hunt_server(cptr, sptr, ":%s SVSNICK %s %s :%s", 1, parc, parv) !=
	HUNTED_ISME)
    {
	if ((acptr = find_person(parv[1], NULL))!=NULL)
	{
	    acptr->umode &= ~UMODE_r;
#ifdef ANTI_NICK_FLOOD
	    acptr->last_nick_change = atoi(parv[3]);
#endif
	    acptr->tsinfo = atoi(parv[3]);
	    sendto_common_channels(acptr, ":%s NICK :%s", parv[1], parv[2]);
	    if (IsPerson(acptr)) add_history(acptr, 1);
	    sendto_serv_butone(NULL, ":%s NICK %s :%i", parv[1], parv[2],
			       acptr->tsinfo);
	    if(acptr->name[0]) del_from_client_hash_table(acptr->name, acptr);
	    strcpy(acptr->name, parv[2]);
	    add_to_client_hash_table(parv[2], acptr);
	}
    }
    return 0;
}

/* channel_svsmode:
 * parv[0] sender
 * parv[1] channel
 * parv[2] modes
 * parv[3] nick
 * parv[4] nickts
 * currently, only a mode of -b is supported.
 * services should use MODE for regular channel modes.
 * 2/5/00 lucas
 * preconditions: parc >= 3, sptr is ulined
 */
int channel_svsmode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aChannel *chptr;
    aClient *acptr = NULL;
    char *m, *nick = NULL;
    char change = '+';
    ts_val nickts = 0;
    int sendmsg = 1;

    if(!(chptr = find_channel(parv[1], NULL)))
	return 0;

    if(parc >= 4)
    {
	nick = parv[3];
	if(parc > 4)
	    nickts = atol(parv[4]);
    }

    if(nick)
    {
	acptr = find_person(nick, NULL);
	if(!acptr || (nickts && acptr->tsinfo != nickts))
	    return 0;
    }

    for(m = parv[2]; *m; m++)
	switch(*m)
	{
	case '+':
	case '-':
            change = *m;
            break;

	case 'b':
            if(nick && MyClient(acptr) && change == '-')
            {
		remove_matching_bans(chptr, acptr, &me);
		sendmsg--;
            }
            break;

	default:
            sendmsg++;
            break;
	}

    if(!sendmsg) return 0;

    if(nick)
	sendto_serv_butone(cptr, ":%s SVSMODE %s %s %s %ld", parv[0], parv[1],
			   parv[2], nick, acptr->tsinfo);
    else
	sendto_serv_butone(cptr, ":%s SVSMODE %s %s", parv[0], parv[1], 
			   parv[2]);

    return 0;
}

/* m_svsmode - df function integrated
 *  - Raistlin
 * -- Behaviour changed - Epi (11/30/99)
 * parv[0] - sender
 * parv[1] - nick
 * parv[2] - TS (or mode, depending on svs version)
 * parv[3] - mode (or services id if old svs version)
 * parv[4] - optional arguement (services id)
 */
int m_svsmode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int            flag, *s, what, setflags;
    char          *m, *modes, *optarg;
    aClient       *acptr;
    ts_val         ts = 0;
#if 0
    int  i;
    char buf[512];

    buf[0] = '\0';
    
    for (i = 0; i < parc; i++) {
	    strcat(buf, parv[i]);
	    strcat(buf, " ");
    }

    sendto_serv_butone(NULL, ":%s GLOBOPS :Got SVSMODE %s", me.name, buf);
    send_globops("from %s: Got SVSMODE %s", me.name, buf);
#endif
    
    if (!IsULine(sptr) || (parc < 3)) {
#if 0
    	sendto_serv_butone(NULL, ":%s GLOBOPS :Got SVSMODE from %s but ignoring because parc < 3", me.name, parv[0]);
	send_globops("from %s: Got SVSMODE from %s but ignoring because parc < 3", me.name, parv[0]);
#endif
	return 0;
    }

    if (parv[1][0] == '#')
	return channel_svsmode(cptr, sptr, parc, parv);

    if ((parc >= 4) && ((parv[3][0] == '+') || (parv[3][0] == '-')))
    {
	ts = atol(parv[2]);
	modes = parv[3];
	optarg = (parc > 4) ? parv[4] : NULL;
    }
    else
    {
	modes = parv[2];
	optarg = (parc > 3) ? parv[3] : NULL;
    }

    if (!(acptr = find_person(parv[1], NULL))) {
#if 0
	sendto_serv_butone(NULL, ":%s GLOBOPS :Got SVSMODE %s for %s but ignoring because cannot find_person()", me.name, modes, parv[1]);
	send_globops("from %s: Got SVSMODE %s for %s but ignoring because cannot find_person()", me.name, modes, parv[1]);
#endif
	return 0;
    }

    if (ts && (ts != acptr->tsinfo)) {
#if 0
	sendto_serv_butone(NULL, ":%s GLOBOPS :Got SVSMODE %s for %s but ignoring because ts %lu does not match acptr->tsinfo %lu", me.name, modes, parv[1], ts, acptr->tsinfo);
	send_globops("from %s: Got SVSMODE %s for %s but ignoring because cannot find_person()", me.name, modes, parv[1], ts, acptr->tsinfo);
#endif
	return 0;
    }

    what = MODE_ADD;
    setflags = 0;

    for (s = user_modes; (flag = *s); s += 2)
	if (acptr->umode & flag)
	    setflags |= flag;
    for (m = modes; *m; m++)
	switch(*m)
	{
	case '+':
	    what = MODE_ADD;
	    break;
	case '-':
	    what = MODE_DEL;
	    break;
	case ' ':
	case '\n':
	case '\r':
	case '\t':
	    break;
	case 'd':
	    if (optarg && isdigit(*optarg))
		acptr->user->servicestamp = strtoul(optarg, NULL, 0);
	    break;
	default:
	    for (s = user_modes; (flag = *s); s += 2)
	    {
		if (*m == (char)(*(s+1)))
		{
		    if (what == MODE_ADD) {
#if 0
			sendto_serv_butone(NULL,
			    ":%s GLOBOPS :Got SVSMODE +%c for %s", me.name,
			    *m, parv[1]);
			send_globops("from %s: Got SVSMODE +%c for %s",
			    me.name, *m, parv[1]);
#endif
			acptr->umode |= flag;
		    } else
			acptr->umode &= ~flag;
		    break;
		}
	    }
	    break;
	}

    if (optarg)
	sendto_serv_butone(cptr, ":%s SVSMODE %s %ld %s %s",
			   parv[0], parv[1], acptr->tsinfo, modes, optarg);
    else
	sendto_serv_butone(cptr, ":%s SVSMODE %s %ld %s",
			   parv[0], parv[1], acptr->tsinfo, modes);

    return 0;
}

/* m_svsjoin - added by dg 11-7-02
 * From unreal 3.1.3
 *   Lamego - Wed Jul 21 20:04:48 1999
 *   Copied off PTlink IRCd (C) PTlink coders team.
 *
 * parv[0] = sender
 * parv[1] = nickname
 * parv[2] = channel
 * parv[3] = key (optional)
 */

int m_svsjoin(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;
    if (!IsULine(sptr))
	return 0;

    if(parc < 3 || !(acptr = find_person(parv[1], NULL)))
	return 0;

    if (MyClient(acptr))
    {
	parv[0] = parv[1];
	parv[1] = parv[2];
	/* key */
	parv[2] = parc > 3 ? parv[3] : NULL;
	(void) m_join(&me, acptr, parc - 1, parv);
    }
    else
    {
	sendto_one(acptr, ":%s SVSJOIN %s %s%s%s", parv[0], parv[1],
		parv[2], 
		/* nasty hackery to make keys behave nicely */
		parc > 3 ? " " : "",
		parc > 3 ? parv[3] : "");
    }

    return 0;
}

